/*
 * cuebreakpoints.c -- print track break points
 *
 * Copyright (C) 2004, 2005, 2006 Svend Sorensen
 * For license terms, see the file COPYING in this distribution.
 *
 * Modified 2005-08-23 by Branden Robinson.
 */

#include <getopt.h>	/* getopt_long() */
#include <stdio.h>	/* fprintf(), printf(), snprintf(), stderr */
#include <stdlib.h>	/* exit() */
#include <string.h>	/* strcasecmp() */
#include "cuefile.h"
#include "time.h"

char *progname;

/*
 * pregap correction modes:
 * APPEND - append pregap to previous track (except for first track)
 * PREPEND - prefix pregap to current track
 * SPLIT - print breakpoints for beginning and end of pregap
 */
enum GapMode {APPEND, PREPEND, SPLIT};

void usage (int status)
{
	char synopsis[1024];

	/* TODO: We could use asprintf() if we know we're using GNU libc. */
	snprintf(synopsis, 1023, "usage: %s [option ...] [file ...]\n",
		 progname);

	if (0 == status) {
		printf("%s", synopsis);
		printf("Report track breakpoints from a CUE or TOC file.\n"
		       "\n"
		       "Options:\n"
		       "-h, --help\t\t\t\tdisplay this message and exit\n"
		       "-i {cue|toc}, --input-format={cue|toc}\tset format of"
		       " input file(s)\n"
		       "--append-gaps\t\t\t\tappend pregaps to previous track\n"
		       "--prepend-gaps\t\t\t\tprefix pregaps to track\n"
		       "--split-gaps\t\t\t\tsplit at beginning and end of"
		       " pregaps\n"
		       "\n"
		       "The default handling of gaps is as if --append-gaps"
		       " were specified.\n"
		       "\n"
		       "See the %s(1) manual page for more information.\n",
		       progname);
	} else {
		fprintf(stderr, "%sRun \"%s --help\" for more information.\n",
			synopsis, progname);
	}

	exit(status);
}

void print_m_ss_ff (long frame)
{
	int m, s, f;

	time_frame_to_msf(frame, &m, &s, &f);
	printf("%d:%02d.%02d\n", m, s, f);
}

void print_breakpoint (long b)
{
	/* do not print zero breakpoints */
	if (0 != b)
		print_m_ss_ff(b);
}

void print_breaks (Cd *cd, int gaps)
{
	int i;
	long b;
	long pg;
	Track *track;

	for (i = 1; i <= cd_get_ntrack(cd); i++) {
		track = cd_get_track(cd, i);
		/*
		 * when breakpoint is at:
		 * index 0: gap is prepended to track
		 * index 1: gap is appended to previous track
		 */
		b = track_get_start(track);
		pg = track_get_index(track, 1) - track_get_zero_pre(track);

		if (gaps == PREPEND || gaps == SPLIT) {
			print_breakpoint(b);
		/* there is no previous track to append the first tracks pregap to */
		} else if (gaps == APPEND && 1 < i) {
			print_breakpoint(b + pg);
		}

		/* if pregap exists, print breakpoints (in split mode) */
		if (gaps == SPLIT && 0 < pg) {
			print_breakpoint(b + pg);
		}
	}
}

int breaks (char *name, int format, int gaps)
{
	Cd *cd = NULL;

	if (NULL == (cd = cf_parse(name, &format))) {
		fprintf(stderr, "%s: error: unable to parse input file"
			" \"%s\"\n", progname, name);
		return -1;
	}

	print_breaks(cd, gaps);

	return 0;
}

int main (int argc, char **argv)
{
	int format = UNKNOWN;
	int gaps = APPEND;
	int ret = 0;		/* return value of breaks() */

	/* option variables */
	int c;
	/* getopt_long() variables */
	extern char *optarg;
	extern int optind;

	static struct option longopts[] = {
		{"help", no_argument, NULL, 'h'},
		{"input-format", required_argument, NULL, 'i'},
		{"append-gaps", no_argument, NULL, 'a'},
		{"prepend-gaps", no_argument, NULL, 'p'},
		{"split-gaps", no_argument, NULL, 's'},
		{NULL, 0, NULL, 0}
	};

	progname = argv[0];

	while (-1 != (c = getopt_long(argc, argv, "hi:", longopts, NULL))) {
		switch (c) {
		case 'h':
			usage(0);
			break;
		case 'i':
			if (0 == strcasecmp("cue", optarg)) {
				format = CUE;
			} else if (0 == strcasecmp("toc", optarg)) {
				format = TOC;
			} else {
				fprintf(stderr, "%s: error: unknown input file"
					" format \"%s\"\n", progname, optarg);
				usage(1);
			}
			break;
		case 'a':
			gaps = APPEND;
			break;
		case 'p':
			gaps = PREPEND;
			break;
		case 's':
			gaps = SPLIT;
			break;
		default:
			usage(1);
			break;
		}
	}

	/* What we do depends on the number of operands. */
	if (optind == argc) {
		/* No operands: report breakpoints of stdin. */
		ret = breaks("-", format, gaps);
	} else {
		/* Report track breakpoints for each operand. */
		for (; optind < argc; optind++) {
			ret = breaks(argv[optind], format, gaps);
			/* Bail out if breaks() returns nonzero. */
			if (!ret)
				break;
		}
	}

	return ret;
}
