/*
 * cueconvert.c -- convert between cue/toc formats
 *
 * Copyright (C) 2004, 2005, 2006 Svend Sorensen
 * For license terms, see the file COPYING in this distribution.
 *
 * Modified 2005-08-23 by Branden Robinson.
 */

#include <getopt.h>	/* getopt_long() */
#include <stdio.h>	/* fprintf(), printf(), snprintf(), stderr */
#include <stdlib.h>	/* exit() */
#include <string.h>	/* strcasecmp() */
#include "cuefile.h"

char *progname;

void usage (int status)
{
	char synopsis[1024];

	/* TODO: We could use asprintf() if we know we're using GNU libc. */
	snprintf(synopsis, 1023, "usage: %s [option ...] [infile [outfile]]\n",
		 progname);

	if (0 == status) {
		printf("%s", synopsis);
		printf("Convert files between CUE and TOC formats.\n"
		       "\n"
		       "Options:\n"
		       "-h, --help\t\t\t\tdisplay this message and exit\n"
		       "-i {cue|toc}, --input-format={cue|toc}\tset format of"
		       " input file\n"
		       "-o {cue|toc}, --output-format={cue|toc}\tset format of"
		       " output file\n"
		       "\n"
		       "See the %s(1) manual page for more information.\n",
		       progname);
	} else {
		fprintf(stderr, "%sRun \"%s --help\" for more information.\n",
				synopsis, progname);
	}

	exit(status);
}

int convert (char *iname, int iformat, char *oname, int oformat)
{
	Cd *cd = NULL;

	if (NULL == (cd = cf_parse(iname, &iformat))) {
		fprintf(stderr, "%s: error: unable to parse input file\n",
			progname);
		return -1;
	}

	if (UNKNOWN == oformat) {
		/* first use file suffix */
		if (UNKNOWN == (oformat = cf_format_from_suffix(oname))) {
			/* then use opposite of input format */
			switch(iformat) {
			case CUE:
					oformat = TOC;
					break;
			case TOC:
					oformat = CUE;
					break;
			}
		}
	}

	return cf_print(oname, &oformat, cd);
}

int main (int argc, char **argv)
{
	int iformat = UNKNOWN;
	int oformat = UNKNOWN;
	int ret = 0;		/* return value of convert() */
	/* getopt_long() variables */
	int c;
	extern char *optarg;
	extern int optind;

	static struct option longopts[] = {
		{"help", no_argument, NULL, 'h'},
		{"input-format", required_argument, NULL, 'i'},
		{"output-format", required_argument, NULL, 'o'},
		{NULL, 0, NULL, 0}
	};

	progname = argv[0];

	while (-1 != (c = getopt_long(argc, argv, "hi:o:", longopts, NULL))) {
		switch (c) {
		case 'h':
			usage(0);
			break;
		case 'i':
			if (0 == strcasecmp("cue", optarg)) {
				iformat = CUE;
			} else if (0 == strcasecmp("toc", optarg)) {
				iformat = TOC;
			} else {
				fprintf(stderr, "%s: unknown input file format"
					" \"%s\"\n", progname, optarg);
				usage(1);
			}
			break;
		case 'o':
			if (0 == strcasecmp("cue", optarg)) {
				oformat = CUE;
			} else if (0 == strcasecmp("toc", optarg)) {
				oformat = TOC;
			} else {
				fprintf(stderr, "%s: unknown output file format"
					" \"%s\"\n", progname, optarg);
				usage(1);
			}
			break;
		default:
			usage(1);
			break;
		}
	}

	/* What we do depends on the number of operands. */
	if (optind == argc) {
		/* No operands: convert stdin to stdout. */
		ret = convert("-", iformat, "-", oformat);
	} else if (optind == argc - 1) {
		/* One operand: convert operand file to stdout. */
		ret = convert(argv[optind], iformat, "-", oformat);
	} else if (optind == argc - 2) {
		/* Two operands: convert input file to output file. */
		ret = convert(argv[optind], iformat, argv[optind + 1], oformat);
	} else {
		usage(1);
	}

	if (0 != ret) {
		fprintf(stderr, "%s: conversion failed\n", progname);
	}

	return ret;
}
